#!/bin/sh

DEST_BASE_DIR=$1
SSCAMERA_FILE="${DEST_BASE_DIR}/Sscamera"
DVA_TASK_FILE="${DEST_BASE_DIR}/DvaTask"
FACE_TASK_FILE="${DEST_BASE_DIR}/FaceTask"
REG_FACE_COUNT_FILE="${DEST_BASE_DIR}/RegisteredFaceCount"
HOME_MODE_FILE="${DEST_BASE_DIR}/HomeMode"
SS_LOG_DB_TARGET="${DEST_BASE_DIR}/log.db"
SS_NOTIFY_MAIL="${DEST_BASE_DIR}/SvsNotifyMail"
SS_NOTIFY_SMS="${DEST_BASE_DIR}/SvsNotifySms"
SS_NOTIFY_PUSH="${DEST_BASE_DIR}/SvsNotifyPush"
SS_NOTIFY_ADV="${DEST_BASE_DIR}/SvsNotifyAdv"

SS_BASE_DIR="/var/packages/SurveillanceStation/target"
SS_BIN_DIR="${SS_BASE_DIR}/bin"
SS_SYSTEM_DB="${SS_BASE_DIR}/system.db"
SS_LOG_DB="${SS_BASE_DIR}/@surveillance/log.db"
SS_SETTING_CONF="/var/packages/SurveillanceStation/etc/settings.conf"

collect_camera() {
    local sscamera="${SS_BIN_DIR}/sscamera -L"

    ${sscamera} > ${SSCAMERA_FILE}
}

collect_dva_task() {
    local table_info_sql="PRAGMA table_info(iva_setting)"
    local grep_no_name="grep -v ^name\$"
    local set_comma_sep="tr '\n' ','"
    local col_list=`sqlite3 ${SS_SYSTEM_DB} "${table_info_sql}" | awk -F '|' '{print $2}' | ${grep_no_name} | ${set_comma_sep}`

    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select ${col_list::-1} from iva_setting" > ${DVA_TASK_FILE}
}

collect_face_task() {
    local table_info_sql="PRAGMA table_info(face_setting)"
    local grep_no_name="grep -v ^name\$"
    local set_comma_sep="tr '\n' ','"
    local col_list=`sqlite3 ${SS_SYSTEM_DB} "${table_info_sql}" | awk -F '|' '{print $2}' | ${grep_no_name} | ${set_comma_sep}`

    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select ${col_list::-1} from face_setting" > ${FACE_TASK_FILE}
}

collect_register_face_count() {
    sudo -u SurveillanceStation psql dva -c "select count(id) from registered_face" > ${REG_FACE_COUNT_FILE}
}

collect_home_mode() {
    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select * from homemodesetting" > ${HOME_MODE_FILE}
}

collect_svs_log() {
    cp ${SS_LOG_DB} ${SS_LOG_DB_TARGET}
}

collect_notification() {
    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select type from notification" > ${SS_NOTIFY_MAIL}
    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select type, server from notificationsms" > ${SS_NOTIFY_SMS}
    sqlite3 ${SS_SYSTEM_DB} ".mode line" "select syno_mail_enable, mobile_enable, attach_snapshot, enable_msg_interval, msg_interval from notificationpushservice" > ${SS_NOTIFY_PUSH}

    local SynoDsmNotify=$(get_key_value "${SS_SETTING_CONF}" "sync_dsm_notify")
	echo "Use DSM notification = ${SynoDsmNotify}" > ${SS_NOTIFY_ADV}

	sqlite3 ${SS_SYSTEM_DB} ".mode line" "select * from notificationadvance" >> ${SS_NOTIFY_ADV}

	local NonAdminNotify=$(get_key_value "${SS_SETTING_CONF}" "non_admin_push_service_mode")
	echo "Notifications for non-admin users = ${NonAdminNotify}" >> ${SS_NOTIFY_ADV}

	local CompanyName=$(get_key_value "${SS_SETTING_CONF}" "company_name")
	echo "%COMPANY_NAME% = ${CompanyName}" >> ${SS_NOTIFY_ADV}
}

mkdir -p ${DEST_BASE_DIR}
collect_camera
collect_dva_task
collect_face_task
collect_register_face_count
collect_home_mode
collect_svs_log
collect_notification
       
exit 0
