#!/usr/bin/env python

import openpyxl
from openpyxl.utils import get_column_letter
import argparse
import sys
import os.path
import json
from numbers import Number

MAX_COLUMN_COUNT = 5

def to_str(input):
    '''
    (bytes == str) in python2
    (bytes != str) in python3
    '''
    if str == type(input):
        return str(input)
    elif bytes == type(input):
        # for python3
        return str(input, 'utf-8')
    else:
        return str(input)

def json2xlsx(src, dst):
    wb = openpyxl.Workbook()
    json_data = {}
    ws = wb.active

    with open(src) as f:
        json_data = json.load(f)

    if 'string' in json_data and len(json_data["string"]):
        ws.merge_cells('A1:E1')
        ws.cell(row=1, column=1).alignment = openpyxl.styles.Alignment(vertical='center')
        ws.cell(row=1, column=1).value = to_str(json_data["string"][0].encode('utf-8'))
        ws.row_dimensions[1].height = 160
        for col in range(1, MAX_COLUMN_COUNT + 1):
            ws.cell(row=2, column=col).font = openpyxl.styles.Font(bold=True)
            ws.cell(row=2, column=col).value = to_str(json_data["string"][col].encode('utf-8'))
            ws.column_dimensions[get_column_letter(col)].width = 35

    row = 3
    for item in json_data["data"]:
        i = 1
        for value in item:
            if not value is None:
                ws.cell(row=row, column=i).value = to_str(value.encode('utf-8'))
            i += 1
        row += 1

    wb.save(filename=dst)

def xlsx2json(src, dst):
    wb = openpyxl.load_workbook(src)
    ws = wb.worksheets[0]
    records=[]

    for row in range(3, ws.max_row + 1):
        array = []
        for col in range(1, MAX_COLUMN_COUNT + 1):
            value = ws.cell(row=row, column=col).value
            if isinstance(value, Number):
                if value == int(value):
                    value = to_str(int(value))
                else:
                    value = to_str(value)
            array.append(value)
        if all(element is None for element in array):
            continue
        records.append(array)

    json.dump(records, open(dst, "w"))

def get_parser():
    parser = argparse.ArgumentParser(description='Convert face json import data')
    parser.add_argument('--src', dest='src_file', type=str, help='Absolute path of input file', required=True)
    parser.add_argument('--dest', dest='dest_file', type=str, help='Absolute path of output file', required=True)

    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    if (not args.src_file.endswith('.json') or not args.dest_file.endswith('.xlsx')) and (not args.src_file.endswith('.xlsx') or not args.dest_file.endswith('.json')):
        print('Error: Wrong input/output file format.')
        parser.print_help()
        sys.exit()

    if args.src_file.endswith('.json'):
        json2xlsx(args.src_file, args.dest_file)
    else:
        xlsx2json(args.src_file, args.dest_file)
