#!/bin/sh
# Copyright (c) 2019 Synology Inc. All rights reserved.

###############################################################
#	This script should be hooked when invoking SYNOShareSet()
#	Usable environment variable:
#		ORIGIN_SHARE_NAME, SHARE_NAME,
#		ORIGIN_SHARE_PATH, SHARE_PATH,
#		ORIGIN_SHARE_ENCRYPTION, SHARE_ENCRYPTION (encrypted/decrypted/no),
#		ORIGIN_SHARE_FORBIDDEN, SHARE_FORBIDDEN (1/0),
#		RESULT
#
#	Note: The script should bear to be invoked multiple times
###############################################################

HOOK_SCRP=/var/packages/SurveillanceStation/target/scripts/ShareHookCommon.sh
. $HOOK_SCRP

PrintArgs()
{
	local Msg="$1"

	SSDebugLog "[$Msg] NAME: $ORIGIN_SHARE_NAME => $SHARE_NAME"
	SSDebugLog "[$Msg] PATH: $ORIGIN_SHARE_PATH => $SHARE_PATH"
	SSDebugLog "[$Msg] ENCRYPTION: $ORIGIN_SHARE_ENCRYPTION => $SHARE_ENCRYPTION"
	SSDebugLog "[$Msg] FORBIDDEN: $ORIGIN_SHARE_FORBIDDEN => $SHARE_FORBIDDEN"
	SSDebugLog "[$Msg] ENABLE(DEPRECATED): $ORIGIN_SHARE_ENABLE => $SHARE_ENABLE"
	SSDebugLog "[$Msg] STATUS(DEPRECATED): $ORIGIN_SHARE_STATUS => $SHARE_STATUS"
	SSDebugLog "[$Msg] RESULT: $RESULT, ToDisable: $(ChgToDisable), IsEnable: $(IsEnable), NeedUpdShare: $(NeedUpdShare)"
}

ChgToDisable()
{
	if [ "$ORIGIN_SHARE_FORBIDDEN" != "$SHARE_FORBIDDEN" -a "$SHARE_FORBIDDEN" == "1" ]; then
		echo true
	else
		echo false
	fi
}

IsEnable()
{
	if [ "$SHARE_FORBIDDEN" == "1" -o "$SHARE_ENCRYPTION" == "encrypted" ]; then
		echo false
	else
		echo true
	fi
}

ChgToUnmount()
{
	# Handle share_set after mount_action unmount
	if [ "$ORIGIN_SHARE_ENCRYPTION" != "$SHARE_ENCRYPTION" -a "$SHARE_ENCRYPTION" == "encrypted" ]; then
		echo true
	else
		echo false
	fi
}

NeedUpdShare()
{
	if [ "$RESULT" != "0" ]; then
		echo false
	elif [ "$ORIGIN_SHARE_PATH" != "$SHARE_PATH" ] ||
		 [ "$ORIGIN_SHARE_FORBIDDEN" != "$SHARE_FORBIDDEN" ] ||
		 [ "true" == $(ChgToUnmount) ]; then
		echo true
	else
		echo false
	fi
}

SSShareSetBegin()
{
	local ToDisable=$(ChgToDisable)

	if [ true == $ToDisable ]; then
		SharePreSetHook true false
	fi
}

SSShareSetEnd()
{
	local Enable=$(IsEnable)

	if [ true == $(NeedUpdShare) ]; then
		SharePostSetHook $Enable
	fi
}

main()
{
	PackageInfoGet;

	case $1 in
		--sdk-mod-ver)
			#Print SDK support version
			echo "2.0"
		;;
		--name)
			#Print package name
			echo ${SURVEILLANCE_PKG_NAME};
		;;
		--pkg-ver)
			#Print package version
			echo ${SURVEILLANCE_PKG_VERSION};
		;;
		--vendor)
			#Print package vendor
			echo ${SURVEILLANCE_PKG_VENDOR};
		;;
		--pre)
			#Actions before share set
			#PrintArgs "--pre"
			SSShareSetBegin;
		;;
		--post)
			#Actions after share set
			#PrintArgs "--post"
			SSShareSetEnd;
		;;
		*)
			echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
	esac
}

main "$@"

