#!/bin/sh
# Copyright (c) 2014 Synology Inc. All rights reserved.

# --sdk-mod-ver
#								SYNOMODULE="volume"
# --name/--pkg-ver/--vendor
#								NONE
# --pre / --post	
#								DEVICE: device (ex: /dev/md1) 
#								MOUNTPOINT: mount point (ex: /volume1) 
#								TYPE: file system type (ex: ext4) 
#								LOCATION: location ("INTERNAL", "EUNIT", "USB", "ESATA")

SURVEILLANCE_INC_SCRP=/var/packages/SurveillanceStation/scripts/SurveillanceStationCommon
. ${SURVEILLANCE_INC_SCRP}

GetActiveVolume()
{
	local ActiveVolume=$(GetValueFromConfFile "${SS_SETTINGS_CONF}" "${CONF_KEY_ACTIVE_VOLUME}")

	if [ -n "$(echo ${ActiveVolume} | grep "@sharebin")" ]; then
		ActiveVolume="$(dirname ${ActiveVolume})"
	fi

	echo "$ActiveVolume"
}

PreUnmount()
{
	if [ false == $(ShouldHandleVolumeEvent) -o \
		 false == $(IsValidSSVolumeLocation "${LOCATION}") ]; then
		return
	fi

	local ActiveVolume="$(GetActiveVolume)"

	if [ ${MOUNTPOINT} == ${ActiveVolume} ]; then
		SetUnmountingActiveVolume true

		if [ true == $(IsModelEDS14) ]; then
			SSDebugLog "Active volume [${ActiveVolume}] is being unmounted. Enter non-recording mode."
			SetNonRecordingMode true volume
		fi
	fi
}

PostUnmount()
{
	local ActiveVolume=$(GetActiveVolume)

	if [ $MOUNTPOINT == "$(GetActiveVolume)" ]; then
		SSDebugLog "Active volume [${ActiveVolume}] is unmounted."
		SetUnmountingActiveVolume false
	fi
}

main()
{
	PackageInfoGet;

	case $1 in
		--sdk-mod-ver)
			echo "1.0"
		;;
		--name)
			echo ${SURVEILLANCE_PKG_NAME}
		;;
		--pkg-ver)
			echo ${SURVEILLANCE_PKG_VERSION}
		;;
		--vendor)
			echo ${SURVEILLANCE_PKG_VENDOR}
		;;
		--pre)
			PreUnmount
		;;
		--post)
			PostUnmount
		;;
		*)
			echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
		;;
	esac
}

main "$@"
