from __future__ import print_function

from nvrTestUtils import SendWebAPI


def AddFaceTask(ip, camId, taskName):
    params = {
        'id': 0,
        'enable': True,
        'blEditMode': False,
        'owner_ds_id': 0,
        'stream_profile': 0,
        'camera_id': camId,
        'name': taskName,
        'similarity': 0.8,
        'display_info': 0,
        'enable_min_obj_size': False,
        'min_obj_size': '0.2',
        'frame_display_info': 1,
        'region_type': '0',
        'det_region_cnt': '1',
        'det_region': '0.0,0.0,0.0,1.0,1.0,1.0,1.0,0.0;0.1000,0.9000,0.1000,0.1000,0.9000,0.1000,0.9000,0.9000;0.2000,0.8000,0.2000,0.2000,0.8000,0.2000,0.8000,0.8000',
        'recognized_color': 3,
        'unrecognized_color': 8,
        'vip_color': 1,
        'vip_list': '',
        'allowed_color': 2,
        'allowed_list': '',
        'blocked_color': 4,
        'blocked_list': '',
        'ignore_bad_quality': True,
        'actFromHost': False
    }
    api = 'SYNO.SurveillanceStation.Face'
    method = 'SaveTask'
    version = 1
    jsonResp = SendWebAPI(ip, api, method, version, params)
    return jsonResp['success']  # is webapi sent successfully


def SendListFaceTasksWebapi(ip):
    api = 'SYNO.SurveillanceStation.Face'
    method = 'ListPlayableTask'
    version = 1
    params = {}
    jsonResp = SendWebAPI(ip, api, method, version, params)
    if not jsonResp or not ['success'] or 'data' not in jsonResp:
        print('Failed to list face tasks on {0}'.format(ip))
        return {}
    return jsonResp


def ListFaceTaskIds(ip):
    jsonResp = SendListFaceTasksWebapi(ip)
    if not jsonResp:
        return []
    taskIds = [jsTask['id'] for jsTask in jsonResp['data']['task']]
    return taskIds


def DeleteFaceTasks(ip, taskIds):
    if not taskIds:
        print('No face tasks to delete, just skip it')
        return True

    taskIdsStr = ','.join([str(taskId) for taskId in taskIds])
    print('Start to delete face tasks of taskIds[{0}]'.format(taskIdsStr))

    api = 'SYNO.SurveillanceStation.Face'
    method = 'DeleteTask'
    version = 1
    params = {
        'ids': taskIdsStr,
        'keepRecording': False
    }
    jsonResp = SendWebAPI(ip, api, method, version, params)
    if 'data' in jsonResp:
        return jsonResp['data']['success']
    return jsonResp['success']


def ClearAllFaceTasks(ip):
    taskIds = ListFaceTaskIds(ip)

    return DeleteFaceTasks(ip, taskIds)
