#!/usr/bin/python

import time
import subprocess
import threading
import json
import os
import pexpect
import sys
import time
import argparse
import ivaUtils
import ivaDefines
import faceUtils

from nvrTestUtils import *

def RunAddRtspCamScript(streamNum, typeIdx, videoReso, fpsIdx):
    scriptCmd = "python addRtspCam.py -n %s -t %s -r %s -f %s" % (str(streamNum), str(typeIdx), videoReso, str(fpsIdx))

    return ShowAndWaitCommand(scriptCmd)

def FindNextFpsSetting(videoConfig, videoType, videoReso, currentFps, minFps):
    count = len(videoConfig['rtspVideoFile'][videoType][videoReso]['data'])

    for fpsIdx in xrange(count - 1, minFps - 2, -1):
        targetFps = videoConfig['rtspVideoFile'][videoType][videoReso]['data'][fpsIdx]['FPS']
        if targetFps <= currentFps:
            return True, fpsIdx

    return False, None


def AddIVATasks(targetDsIp, NumIvaTasks, IVATaskType, isClearExisting=True):
    if isClearExisting and not ivaUtils.ClearAllIVATasks(targetDsIp):
        print('Failed to clear existing iva tasks')
        return False

    ret, camIdListStr = GetCamIdListStr(targetDsIp, None)
    if not ret:
        print('Failed to list camIds')
        return False

    camIdList = map(int, camIdListStr.split(','))
    ivaTaskNum = min(NumIvaTasks, len(camIdList))

    for i in xrange(ivaTaskNum):
        if not ivaUtils.AddIVATask(targetDsIp, camIdList[i],
                                   IVATaskType,
                                   'performance_{0}'.format(i)):
            print('Failed to add iva task of camId {0}'.format(camIdList[i]))
            return False
    print('{0} IVA tasks Added'.format(ivaTaskNum))
    return True


def AddFaceTasks(targetDsIp, numFaceTasks, isClearExisting=True):
    if isClearExisting and not faceUtils.ClearAllFaceTasks(targetDsIp):
        print('Failed to clear existing face tasks')
        return False

    ret, camIdListStr = GetCamIdListStr(targetDsIp, None)
    if not ret:
        print('Failed to list camIds')
        return False

    camIdList = map(int, camIdListStr.split(','))
    faceTaskNum = min(numFaceTasks, len(camIdList))

    for i in xrange(faceTaskNum):
        if not faceUtils.AddFaceTask(targetDsIp, camIdList[i], 'performance_{0}'.format(i)):
            print('Failed to add face task of camId {0}'.format(camIdList[i]))
            return False

    print('{0} Face tasks Added'.format(faceTaskNum))

    return True


def GetTotalFps(config, videoConfig, rtspConfig):
    typeCount = len(videoConfig['videoType']['data'])
    resultCamFpsMap = {}
    resultCondMap = {}
    targetDsIp = config['targetDs']['ip']

    try:
        for typeIdx in xrange(0, typeCount):
            videoType = videoConfig['videoType']['data'][typeIdx]
            resultCamFpsMap[videoType] = {}
            resultCondMap[videoType] = {}

            if videoType not in rtspConfig['totalFpsSetting']['FpsBound']:
                continue

            for resolutionItem in rtspConfig['totalFpsSetting']['FpsBound'][videoType]['data']:
                currentFps = resolutionItem['FPS']
                minFps = max(resolutionItem.get('minFPS', DEF_MIN_FPS), DEF_MIN_FPS)
                resolution = resolutionItem['resolution']
                startCam = resolutionItem.get('maxCam', rtspConfig['totalFpsSetting']['maxCam'])
                currentCam = startCam
                blEnterFps10 = False

                # H. 264, 720P has already been performed in maxCamSearch.py
                '''
                if "H.264" == videoType and "720P" == resolution:
                    continue
                '''

                if "MJPEG" == videoType and "4K" == resolution:
                    continue

                while True:
                    if False is blEnterFps10:
                        res, fpsIdx = FindNextFpsSetting(videoConfig, videoType, resolution, currentFps, minFps)
                    else:
                        currentCam -= 1
                        res = (0 < currentCam)

                    if False is res:   # searching ends with no success record
                        resultCamFpsMap[videoType][resolution] = None
                        resultCondMap[videoType][resolution] = None
                        break

                    currentFps = videoConfig['rtspVideoFile'][videoType][resolution]['data'][fpsIdx]['FPS']

                    if DEF_MIN_FPS == currentFps and False is blEnterFps10:
                        blEnterFps10 = True
                        currentCam = resolutionItem.get('CHtoDecreaseFromWhenFps10', startCam)

                    if -1 == AddCamera(currentCam, currentFps, typeIdx, resolution):
                        print('Failed to add cameras')
                        return resultCamFpsMap, resultCondMap

                    if config['monitorParams']['monitorGPU']:
                        ivaTaskNum = config['ivaSetting']['taskNum']
                        ivaTaskType = config['ivaSetting']['taskType']
                        isIvaTasksAdded =  AddIVATasks(targetDsIp, ivaTaskNum, ivaTaskType)

                        isFaceTasksAdded = AddFaceTasks(targetDsIp, config['faceSetting']['taskNum'])

                        if not isIvaTasksAdded or not isFaceTasksAdded:
                            return resultCamFpsMap, resultCondMap

                    simRes = RunSimulation(resultCamFpsMap, resultCondMap, config, currentCam, currentFps,
                                           videoType, resolution)
                    if -1 == simRes:   # error happens
                        return resultCamFpsMap, resultCondMap
                    elif 1 == simRes:  # result found
                        break
                    else:
                        currentFps -= 1

        return resultCamFpsMap, resultCondMap

    except KeyboardInterrupt:
        resultCamFpsMap[videoType][resolution] = None
        resultCondMap[videoType][resolution] = None
        return resultCamFpsMap, resultCondMap

def AddCamera(currentCam, currentFps, typeIdx, resolution):
    res = RunAddRtspCamScript(currentCam, typeIdx, resolution, currentFps)
    if False is res:
        print("Add camera failed. Check add camera script.")
        return -1

def RunSimulation(camFpsMap, condMap, config, currentCam, currentFps, videoType, resolution):
    print("======== current run: resolution %s FPS %d Camera %d =======" % (resolution, currentFps, currentCam))
    logName = "nvr_performance_totalFps_{0}_{1}_{2}_{3}.log".format(videoType, resolution, currentFps, currentCam)
    monitorResult, condValList = Simulation(config, currentCam, logName)

    if True is GetSignalStatus():
        print("Searching total fps failed.")
        return -1

    if True is monitorResult:
        print("======== %s %s with FPS %d Camera %d is passed ========" %(videoType, resolution, currentFps, currentCam))
        camFpsMap[videoType][resolution] = (currentCam, currentFps)
        condMap[videoType][resolution] = condValList
        return 1

    return 0

def PrintResult(config, videoConfig, fpsMap, condMap, maxCam):
    headItem = ["Video type", "Resolution", "cameras", "FPS", "Total FPS",
                "CPU Avg. %", "Mem %", "Webapi", "Recording", "Liveview"]

    if config["monitorParams"]["monitorGPU"]:
        headItem.extend(('GPU Mem %', 'GPU temp'))

    print("\nTotal FPS result")
    print(' '.join(['{0:>12}'.format(x) for x in headItem]))

    for videoType in videoConfig['videoType']['data']:
        if videoType not in fpsMap:
            continue

        for resolutionItem in rtspConfig['totalFpsSetting']['FpsBound'][videoType]['data']:
            resolution = resolutionItem['resolution']
            if resolution not in fpsMap[videoType]:
                continue

            resultCamFps = fpsMap[videoType][resolution]
            if None != resultCamFps:
                camNum = resultCamFps[0]
                fps = resultCamFps[1]
                sys.stdout.write('{0:>12} {1:>12} {2:>12} {3:>12} {4:>12} '.format(videoType, resolution, camNum, fps, camNum * fps))
            else:
                sys.stdout.write('{0:>12} {1:>12} {2:>12} {3:>12} {4:>12} '.format(videoType, resolution, "not found", "not found", "0"))

            if None != condMap[videoType][resolution]:
                for value in condMap[videoType][resolution]:
                    sys.stdout.write('{0:>12} '.format(value))
            print('')

def FetchConfig():
    with open('conf.json') as jsonData:
        config = json.load(jsonData)

    with open('videoInfo.json') as jsonData:
        videoConfig = json.load(jsonData)

    with open('rtspConf.json') as jsonData:
        rtspConfig = json.load(jsonData)

    return config, videoConfig, rtspConfig

def GenCookieFile(ip, user, passwd):
    authRes, sid = AuthDsAndGetSid(ip, user, passwd)
    if False is authRes:
        print('Failed to generate cookie files of {0}, please make sure you have entered the right username/passwd')
    return authRes

def GenCookieFiles(rtspConfig):
    rtsp_list = rtspConfig['rtspServer']['data']

    for rtsp in rtsp_list:
        if False is GenCookieFile(rtsp['ip'], rtsp['username'], rtsp['password']):
            return False

    target = rtspConfig['targetDs']
    return GenCookieFile(target['ip'], target['username'], target['password'])

if __name__ == "__main__":
    config, videoConfig, rtspConfig = FetchConfig()

    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--enable_scp", dest='scpEnable', action='store_true',
                        help = "set this option to skip copying files to target/rtsp/viewer")
    parser.add_argument("-ncr", "--disable_check_resync", dest='checkResync', action='store_false',
						help = "disable check resync")

    args = parser.parse_args()

    EnableTargetDsSSHRootLogin(config, args.checkResync)
    EnableViewerSSHRootLogin(config)
    EnableRtspSSHRootLogin(rtspConfig)
    EditMaxCameraConf(config)

    if True is args.scpEnable:
        if False is AddAccountAndScpFile(config):
            sys.exit(1)

        if False is RunScpRtspVideoDirCommand():
            sys.exit(1)

    if False is GenCookieFiles(rtspConfig):
        sys.exit(1)
    print('Cookie files generated')

    camFpsMap, condMap = GetTotalFps(config, videoConfig, rtspConfig)
    PrintResult(config, videoConfig, camFpsMap, condMap, rtspConfig['totalFpsSetting']['maxCam'])

    sys.exit()
